package com.link.core.basic.config;

import com.link.core.util.CoreYmlConstants;
import com.link.core.util.StringUtils;
import com.link.core.util.dynamicredis.DynamicRedisProps;
import java.util.List;
import javax.annotation.Resource;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.redisson.Redisson;
import org.redisson.api.RedissonClient;
import org.redisson.config.ClusterServersConfig;
import org.redisson.config.Config;
import org.redisson.config.SentinelServersConfig;
import org.redisson.config.SingleServerConfig;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.DependsOn;
import org.springframework.core.env.Environment;

@Configuration
public class RedissonConfig implements EnvironmentAware {
   private static boolean clusterFlag;
   private static String cluster;
   private static String auth;
   private static int timeOut;
   private static int MIN_IDLE;
   private static int MAX_IDLE = 20000;
   private static int MAX_WAIT = 1000000;
   private static int MAX_POOL = 102400;
   private static final Logger logger = LogManager.getLogger(RedissonConfig.class);
   @Resource
   private DynamicRedisProps dynamicRedisProps;

   public void setEnvironment(Environment environment) {
      String redisClusterKey = "spring.redis.cluster.state";
      String clusterFlagStr = environment.getProperty("spring.redis.cluster.state");
      if ("true".equals(clusterFlagStr)) {
         clusterFlag = true;
      }

      String currentRedisName = environment.getProperty("dynamicredis.currentname");
      if (StringUtils.isBlank(currentRedisName)) {
         String clusterKey = "spring.redis.cluster.nodes";
         cluster = environment.getProperty("spring.redis.cluster.nodes");
         String authKey = "spring.redis.cluster.password";
         auth = environment.getProperty("spring.redis.cluster.password");
      } else {
         List<String> names = this.dynamicRedisProps.getName();

         for(int i = 0; i < names.size(); ++i) {
            if (currentRedisName.equals(names.get(i))) {
               cluster = (String)this.dynamicRedisProps.getNodes().get(i);
               auth = (String)this.dynamicRedisProps.getPassword().get(i);
            }
         }
      }

      logger.info("redisson连接的节点信息：" + cluster);
      logger.info("redisson连接的认证信息：" + auth);
   }

   @Bean
   @DependsOn({"coreYmlProperties"})
   RedissonClient getMemberRedisson() {
      RedissonClient redissonClient;
      if (clusterFlag) {
         redissonClient = getClusterRedissonClient();
      } else {
         redissonClient = getSingleRedissonClient();
      }

      return redissonClient;
   }

   private static RedissonClient getClusterRedissonClient() {
      Config config = new Config();
      String[] nodes = cluster.split(",");

      for(int i = 0; i < nodes.length; ++i) {
         nodes[i] = "redis://" + nodes[i];
      }

      ((ClusterServersConfig)((ClusterServersConfig)((ClusterServersConfig)config.useClusterServers().setScanInterval(2000).addNodeAddress(nodes).setPassword(auth)).setTimeout(MAX_WAIT)).setMasterConnectionMinimumIdleSize(500)).setMasterConnectionPoolSize(500);
      RedissonClient redisson = Redisson.create(config);
      return redisson;
   }

   private static RedissonClient getSentinelRedissonClient() {
      Config config = new Config();
      String address = CoreYmlConstants.getApplicationRedisHost() + ":" + CoreYmlConstants.getApplicationRedisPort();
      String auth = CoreYmlConstants.getApplicationRedisPassword();
      String masterName = CoreYmlConstants.getApplicationRedisClusterMasterName();
      SentinelServersConfig serverConfig = (SentinelServersConfig)config.useSentinelServers().addSentinelAddress(new String[]{address}).setMasterName(masterName).setTimeout(MAX_WAIT);
      if (auth != null && !"".equals(auth)) {
         serverConfig.setPassword(auth);
      }

      return Redisson.create(config);
   }

   private static RedissonClient getSingleRedissonClient() {
      Config config = new Config();
      String address = "redis://" + CoreYmlConstants.getApplicationRedisHost() + ":" + CoreYmlConstants.getApplicationRedisPort();
      String auth = CoreYmlConstants.getApplicationRedisPassword();
      SingleServerConfig serverConfig = ((SingleServerConfig)config.useSingleServer().setAddress(address).setTimeout(MAX_WAIT)).setConnectionPoolSize(MAX_POOL).setConnectionMinimumIdleSize(MIN_IDLE);
      if (auth != null && !"".equals(auth)) {
         serverConfig.setPassword(auth);
      }

      RedissonClient redissonClient = Redisson.create(config);
      return redissonClient;
   }
}
